import tempfile
import os
from typing import Optional
from fastapi import APIRouter, Body
from pydantic import BaseModel
from tcheckerpy.utils import call_tchecker
import asyncio

router = APIRouter(prefix="/tck_reach", tags=["tck_reach"])


class TckReachBody(BaseModel):
    sysdecl: str
    labels: str
    algorithm: int
    search_order: str
    certificate: int
    block_size: Optional[int] = None
    table_size: Optional[int] = None

@router.put("")
async def reach(body: TckReachBody = Body(...)):
    
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.sysdecl.encode('utf-8'))
        temp_file_path = temp_file.name
        
    # print(temp_file_path)
    # print(body.labels)

    # Call the TChecker reachability function with following definition:
    # void tck_reach(const char * output_filename, const char * sysdecl_filename, const char * labels,
    #            tck_reach_algorithm_t algorithm, const char * search_order, tck_reach_certificate_t certificate,
    #            int * block_size, int * table_size);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True

    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_reach",
        argtypes=["ctypes.c_char_p", "ctypes.c_char_p", "ctypes.c_int", "ctypes.c_char_p", "ctypes.c_int", "ctypes.POINTER(ctypes.c_int)", "ctypes.POINTER(ctypes.c_int)"],
        has_result=True,
        args=[temp_file_path, body.labels, body.algorithm, body.search_order, body.certificate, body.block_size, body.table_size]
    )

    # Cleanup
    os.remove(temp_file_path)

    resultMap = {
        "stats": output,
        "certificate": result
    }
    # print("Output: " + output)
    
    return resultMap



import tempfile
from typing import Optional
from fastapi import APIRouter, Body, HTTPException, Response
from fastapi.responses import JSONResponse
from pydantic import BaseModel
from tcheckerpy.utils import call_tchecker

router = APIRouter(prefix="/tck_simulate", tags=["tck_simulate"])


class TCKSimulationRequest(BaseModel):
    sysdecl: str
    simulation_type: int
    starting_state: Optional[str] = None
    nsteps: Optional[int] = None


@router.put("/simulate")
async def simulate_tck(
    body: TCKSimulationRequest = Body(..., description="Request body for TCK simulation")
):
    if not body.sysdecl:
        raise HTTPException(status_code=422, detail="sysdecl cannot be empty")

    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.sysdecl.encode('utf-8'))
        temp_file_path = temp_file.name

    # Call the TChecker simulation function with following definition:
    # void tck_simulate(const char * output_filename, const char * sysdecl_filename, simulation_type_t simulation_type,
    #                   const tchecker::simulate::display_type_t display_type, const char * starting_state_attributes, int nsteps,
    #                   bool output_trace);  
    # output_filename is set by the call function when has_result=True

    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_simulate",
        argtypes=["ctypes.c_char_p", "ctypes.c_int", "ctypes.c_int", "ctypes.c_char_p", "ctypes.POINTER(ctypes.c_int)", "ctypes.c_bool"],
        has_result=True,
        args=[temp_file_path, body.simulation_type, 1, body.starting_state or "", body.nsteps or 0, False] 
    )
    #remove last newline character and quotes from result
    result = result.strip()

    # print("Output: " + output)
    # print("Result: " + result)


    return Response(content=result, media_type="application/json")


    



import tempfile
import os
from typing import Optional
from fastapi import APIRouter, Body, HTTPException
from pydantic import BaseModel
from tcheckerpy.utils import call_tchecker
import asyncio

router = APIRouter(prefix="/tck_liveness", tags=["tck_liveness"])


class TckLivenessBody(BaseModel):
    sysdecl: str
    labels: str
    algorithm: int
    certificate: int
    block_size: Optional[int] = None
    table_size: Optional[int] = None

@router.put("")
async def liveness(body: TckLivenessBody = Body(...)):

    if not body or not body.sysdecl:
        raise HTTPException(status_code=422, detail="Request body and sysdecls cannot be empty")

    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.sysdecl.encode('utf-8'))
        temp_file_path = temp_file.name
        
    # print(temp_file_path)
    # print(body.labels)


    # Call the TChecker liveness function with following definition:
    # void tck_liveness(const char * output_filename, const char * sysdecl_filename, const char * labels,
    #               tck_liveness_algorithm_t algorithm, tck_liveness_certificate_t certificate, int * block_size,
    #               int * table_size);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True
    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_liveness",
        argtypes=["ctypes.c_char_p", "ctypes.c_char_p", "ctypes.c_int", "ctypes.c_int", "ctypes.POINTER(ctypes.c_int)", "ctypes.POINTER(ctypes.c_int)"],
        has_result=True,
        args=[temp_file_path, body.labels, body.algorithm, body.certificate, body.block_size, body.table_size]
    )

    # Cleanup
    os.remove(temp_file_path)

    resultMap = {
        "stats": output,
        "certificate": result
    }
    # print("Output: " + output)
    
    return resultMap



import tempfile
import os
from fastapi import APIRouter, Body, HTTPException
from pydantic import BaseModel
from tcheckerpy.utils import call_tchecker

router = APIRouter(prefix="/tck_syntax", tags=["tck_syntax"])

@router.put("/check", summary="Check syntax of a timed automaton")
def check(body: str = Body(...)):

    if not body or body.strip() == "":
        raise HTTPException(status_code=422, detail="Request body cannot be empty")
    
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.encode('utf-8'))
        temp_file_path = temp_file.name
        
    # print(temp_file_path)



    # Call the TChecker syntax check function with following definition:
    # void tck_syntax_check_syntax(const char * output_filename, const char * sysdecl_filename);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True
    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_syntax_check_syntax",
        argtypes=["ctypes.c_char_p"],
        has_result=True,
        args=[temp_file_path]
    )

    #Cleanup
    os.remove(temp_file_path)

    #remove last newline and trailing spaces
    result = result.strip()

    if result.endswith("Syntax OK"):
        result = {"status": "success", "message": "Syntax is correct"}
    else:
        result = {"status": "error", "message": result}
    return result

@router.put("/to_dot", summary="Convert timed automaton to DOT format")
def to_dot(body: str = Body(...)):

    if not body or body.strip() == "":
        raise HTTPException(status_code=422, detail="Request body cannot be empty")
    
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.encode('utf-8'))
        temp_file_path = temp_file.name
        
    
    # Call the TChecker syntax to DOT conversion function with following definition:
    # void tck_syntax_to_dot(const char * output_filename, const char * sysdecl_filename);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True
    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_syntax_to_dot",
        argtypes=["ctypes.c_char_p"],
        has_result=True,
        args=[temp_file_path]
    )
    # Cleanup
    os.remove(temp_file_path)
        
    return result

@router.put("/to_json", summary="Convert timed automaton to JSON format")
def to_json(body: str = Body(...)):

    if not body or body.strip() == "":
        raise HTTPException(status_code=422, detail="Request body cannot be empty")
    
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.encode('utf-8'))
        temp_file_path = temp_file.name
        
    
    # Call the TChecker syntax to JSON conversion function with following definition:
    # void tck_syntax_to_json(const char * output_filename, const char * sysdecl_filename);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True
    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_syntax_to_json",
        argtypes=["ctypes.c_char_p"],
        has_result=True,
        args=[temp_file_path]
    )

    # print(output)
    # Cleanup
    os.remove(temp_file_path)
        
    return result


class CreateSynchronizedProductBody(BaseModel):
    sysdecl: str
    process_name: str

@router.put("/create_synchronized_product", summary="Create a synchronized product of timed automata")
def create_product(body: CreateSynchronizedProductBody = Body(...)):

    if not body:
        raise HTTPException(status_code=422, detail="Request body cannot be empty")
    
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(body.sysdecl.encode('utf-8'))
        temp_file_path = temp_file.name
        
    
    # Call the TChecker create synchronized product function with following definition:
    # void tck_syntax_create_synchronized_product(const char * output_filename, const char * sysdecl_filename, const char * new_system_name);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True
    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_syntax_create_synchronized_product",
        argtypes=["ctypes.c_char_p", "ctypes.c_char_p"],
        has_result=True,
        args=[temp_file_path, body.process_name]
    )

    # Cleanup
    os.remove(temp_file_path)
        
    return result



import ctypes
import tempfile
import os
from typing import Optional
from fastapi import APIRouter, Body, HTTPException
from pydantic import BaseModel
from tcheckerpy.utils import call_tchecker
import asyncio

router = APIRouter(prefix="/tck_compare", tags=["tck_compare"])


class TckCompareBody(BaseModel):
    first_sysdecl: str
    second_sysdecl: str
    relationship: int
    block_size: Optional[int] = None
    table_size: Optional[int] = None

@router.put("")
async def compare(body: TckCompareBody = Body(...)):

    if not body or not body.first_sysdecl or not body.second_sysdecl:
        raise HTTPException(status_code=422, detail="Request body and sysdecls cannot be empty")
    
    with tempfile.NamedTemporaryFile(delete=False) as temp_file_first_sysdecl:
        temp_file_first_sysdecl.write(body.first_sysdecl.encode('utf-8'))
        temp_file_path_first_sysdecl = temp_file_first_sysdecl.name
    with tempfile.NamedTemporaryFile(delete=False) as temp_file_second_sysdecl:
        temp_file_second_sysdecl.write(body.second_sysdecl.encode('utf-8'))
        temp_file_path_second_sysdecl  = temp_file_second_sysdecl.name
        
    
    # Call the TChecker compare function with following definition:
    # void tck_compare(const char * output_filename, const char * first_sysdecl_filename, const char * second_sysdecl_filename,
    #              tck_compare_relationship_t relationship, int * block_size, int * table_size);
    # output_filename is not included in the argtypes as it is set by the call function when has_result=True
    output, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_compare",
        argtypes=["ctypes.c_char_p", "ctypes.c_char_p", "ctypes.c_int", "ctypes.POINTER(ctypes.c_int)", "ctypes.POINTER(ctypes.c_int)"],
        has_result=True,
        args=[temp_file_path_first_sysdecl, temp_file_path_second_sysdecl, body.relationship, body.block_size, body.table_size]
    )

    # Cleanup
    os.remove(temp_file_path_first_sysdecl)
    os.remove(temp_file_path_second_sysdecl)

    resultMap = {
        "stats": output,
    }
    # print("Output: " + output)
    
    return resultMap
